import json
import os
import pytest
import sys

_this_dir = os.path.dirname(os.path.abspath(__file__))
_opencc_rootdir = os.path.abspath(os.path.join(_this_dir, '..', '..'))
_testcases_path = os.path.join(_opencc_rootdir, 'test', 'testcases', 'testcases.json')


def test_import():
    import opencc  # noqa


def test_init_delete_converter():
    import opencc

    for config in opencc.CONFIGS:
        converter = opencc.OpenCC(config)
        del converter


def test_conversion():
    import opencc

    with open(_testcases_path, 'r', encoding='utf-8') as f:
        parsed = json.load(f)

    for case in parsed.get('cases', []):
        input_text = case.get('input')
        expected = case.get('expected', {})
        if not input_text or not isinstance(expected, dict):
            continue
        for cfg, ans in expected.items():
            converter = opencc.OpenCC(f'{cfg}.json')
            assert converter.convert(input_text) == ans, \
                'Failed to convert {} for {} -> {}'.format(cfg, input_text, ans)


if __name__ == "__main__":
    sys.exit(pytest.main(sys.argv[1:]))



# -*- coding: utf-8 -*-

import codecs
import sys


def sort_items(input_filename, output_filename):
    input_file = codecs.open(input_filename, "r", encoding="utf-8")

    lines = [line.rstrip("\r\n") for line in input_file]
    input_file.close()

    def line_type(line):
        if line == "" or line.strip() == "":
            return "empty"
        if line.startswith("#"):
            return "comment"
        if "\t" in line:
            return "entry"
        raise ValueError("Invalid dictionary line: " + line)

    parsed = []
    for line in lines:
        parsed.append({"type": line_type(line), "content": line})

    entry_lines = [i for i, p in enumerate(parsed) if p["type"] == "entry"]
    if not entry_lines:
        header_blocks = []
        current = []
        for p in parsed:
            if p["type"] == "comment":
                current.append(p["content"])
            elif p["type"] == "empty":
                if current:
                    header_blocks.append(list(current))
                    current = []
        if current:
            header_blocks.append(list(current))

        output_file = open(output_filename, "wb")
        for idx, block in enumerate(header_blocks):
            for line in block:
                output_file.write((line + "\n").encode("utf-8"))
            if idx < len(header_blocks) - 1:
                output_file.write(b"\n")
        if header_blocks:
            output_file.write(b"\n")
        output_file.close()
        return

    first_entry = entry_lines[0]
    last_entry = entry_lines[-1]

    header_end = -1
    for i in range(first_entry - 1, -1, -1):
        if parsed[i]["type"] == "empty":
            header_end = i
            break

    header_blocks = []
    current = []
    for i in range(0, header_end + 1):
        if parsed[i]["type"] == "comment":
            current.append(parsed[i]["content"])
        elif parsed[i]["type"] == "empty":
            if current:
                header_blocks.append(list(current))
                current = []
    if current:
        header_blocks.append(list(current))

    footer_blocks = []
    current = []
    for i in range(last_entry + 1, len(parsed)):
        if parsed[i]["type"] == "comment":
            current.append(parsed[i]["content"])
        elif parsed[i]["type"] == "empty":
            if current:
                footer_blocks.append(list(current))
                current = []
    if current:
        footer_blocks.append(list(current))

    annotated_entries = []
    floating_blocks = []
    current = []
    entry_index = 0
    for i in range(header_end + 1, last_entry + 1):
        p = parsed[i]
        if p["type"] == "comment":
            current.append(p["content"])
            continue
        if p["type"] == "empty":
            if current:
                floating_blocks.append({"anchor": entry_index, "lines": list(current)})
                current = []
            continue
        if p["type"] == "entry":
            attached = None
            if current:
                has_empty = False
                for j in range(i - 1, -1, -1):
                    if parsed[j]["type"] == "entry":
                        break
                    if parsed[j]["type"] == "empty":
                        has_empty = True
                        break
                if has_empty:
                    floating_blocks.append({"anchor": entry_index, "lines": list(current)})
                else:
                    attached = list(current)
                current = []

            key, value = p["content"].split("\t", 1)
            annotated_entries.append(
                {
                    "key": key,
                    "value": value,
                    "attached": attached,
                    "original_index": entry_index,
                }
            )
            entry_index += 1

    if current:
        floating_blocks.append({"anchor": entry_index, "lines": list(current)})

    annotated_entries.sort(key=lambda e: e["key"])
    index_map = {e["original_index"]: i for i, e in enumerate(annotated_entries)}
    for block in floating_blocks:
        if block["anchor"] in index_map:
            block["anchor"] = index_map[block["anchor"]]
        else:
            block["anchor"] = len(annotated_entries)

    floating_by_anchor = {}
    for block in floating_blocks:
        floating_by_anchor.setdefault(block["anchor"], []).append(block["lines"])

    output_file = open(output_filename, "wb")

    for idx, block in enumerate(header_blocks):
        for line in block:
            output_file.write((line + "\n").encode("utf-8"))
        if idx < len(header_blocks) - 1:
            output_file.write(b"\n")
    if header_blocks and annotated_entries:
        output_file.write(b"\n")

    for i, entry in enumerate(annotated_entries):
        for block in floating_by_anchor.get(i, []):
            output_file.write(b"\n")
            for line in block:
                output_file.write((line + "\n").encode("utf-8"))
            output_file.write(b"\n")

        if entry["attached"]:
            for line in entry["attached"]:
                output_file.write((line + "\n").encode("utf-8"))
        output_file.write(
            (entry["key"] + "\t" + entry["value"] + "\n").encode("utf-8")
        )

    for block in floating_by_anchor.get(len(annotated_entries), []):
        output_file.write(b"\n")
        for line in block:
            output_file.write((line + "\n").encode("utf-8"))

    if footer_blocks:
        if annotated_entries:
            output_file.write(b"\n")
        for idx, block in enumerate(footer_blocks):
            for line in block:
                output_file.write((line + "\n").encode("utf-8"))
            if idx < len(footer_blocks) - 1:
                output_file.write(b"\n")

    output_file.close()


def reverse_items(input_filename, output_filename):
    input_file = codecs.open(input_filename, "r", encoding="utf-8")
    dic = {}

    for line in input_file:
        stripped = line.strip()
        if not stripped or stripped.startswith("#"):
            continue
        key, value = line.split("\t")
        while value[-1] == "\n" or value[-1] == "\r":
            value = value[:-1]

        value_list = value.split(" ")
        for value in value_list:
            if value in dic:
                dic[value].append(key)
            else:
                dic[value] = [key]

    input_file.close()

    output_file = open(output_filename, "wb")

    for key in sorted(dic.keys()):
        line = key + "\t" + " ".join(dic[key]) + "\n"
        output_file.write(line.encode('utf-8'))

    output_file.close()


def find_target_items(input_filename, keyword):
    input_file = codecs.open(input_filename, "r", encoding="utf-8")
    for line in input_file:
        stripped = line.strip()
        if not stripped or stripped.startswith("#"):
            continue
        key, value = line.split("\t")
        while value[-1] == "\n" or value[-1] == "\r":
            value = value[:-1]

        value_list = value.split(" ")
        for value in value_list:
            if keyword in value:
                sys.stdout.write(line)

    input_file.close()



import os
import re
import shutil
import subprocess
import sys
import warnings

import setuptools
import setuptools.command.build_ext
import wheel.bdist_wheel

_this_dir = os.path.dirname(os.path.abspath(__file__))
_build_dir = os.path.join(_this_dir, 'build', 'python')

_cmake_file = os.path.join(_this_dir, 'CMakeLists.txt')
_author_file = os.path.join(_this_dir, 'AUTHORS')
_readme_file = os.path.join(_this_dir, 'README.md')
_fallback_version = '1.3.0'


def _get_version_from_git():
    try:
        raw = subprocess.check_output(
            ['git', 'describe', '--tags', '--long', '--always'],
            cwd=_this_dir,
            stderr=subprocess.DEVNULL,
        ).decode('utf-8').strip()
    except (OSError, subprocess.CalledProcessError):
        return ''

    dirty = ''
    for diff_cmd in (['git', 'diff', '--quiet'], ['git', 'diff', '--cached', '--quiet']):
        try:
            subprocess.check_call(
                diff_cmd,
                cwd=_this_dir,
                stdout=subprocess.DEVNULL,
                stderr=subprocess.DEVNULL,
            )
        except (OSError, subprocess.CalledProcessError):
            dirty = '.dirty'
            break

    release_match = re.match(r'^(?:v|ver\.)(\d+\.\d+\.\d+)-0-g[0-9a-f]+$', raw)
    if release_match:
        return '{}{}'.format(release_match.group(1), dirty)

    dev_match = re.match(r'^(?:v|ver\.)(\d+\.\d+\.\d+)-(\d+)-g([0-9a-f]+)$', raw)
    if dev_match:
        return '{}.dev{}+g{}{}'.format(
            dev_match.group(1),
            dev_match.group(2),
            dev_match.group(3),
            dirty,
        )

    return '{}+g{}{}'.format(_fallback_version, raw, dirty)


def get_version_info():
    # Prefer Git-derived version from environment (set by CI or compute-version.sh)
    env_version = os.environ.get('VERSION', '')
    if env_version:
        # Strip leading 'v' prefix for PEP 440 compatibility
        return env_version.lstrip('v')

    git_version = _get_version_from_git()
    if git_version:
        return git_version

    return _fallback_version


def get_author_info():
    if not os.path.isfile(_author_file):
        return 'BYVoid', 'byvoid@byvoid.com'

    authors = []
    emails = []
    author_pattern = re.compile(r'(.+) <(.+)>')
    with open(_author_file, 'rb') as f:
        for line in f:
            match = author_pattern.search(line.decode('utf-8'))
            if not match:
                continue
            authors.append(match.group(1))
            emails.append(match.group(2))

    if len(authors) == 0:
        return 'BYVoid', 'byvoid@byvoid.com'

    return ', '.join(authors), ', '.join(emails)


def get_long_description():
    with open(_readme_file, 'rb') as f:
        return f.read().decode('utf-8')


def build_libopencc(output_path):
    print('building libopencc into %s' % _build_dir)

    is_windows = sys.platform == 'win32'

    # Make build directories
    os.makedirs(_build_dir, exist_ok=True)

    # Configure
    cmake_args = [
        '-DBUILD_DOCUMENTATION:BOOL=OFF',
        '-DBUILD_SHARED_LIBS:BOOL=OFF',
        '-DENABLE_GTEST:BOOL=OFF',
        '-DENABLE_BENCHMARK:BOOL=OFF',
        '-DBUILD_PYTHON:BOOL=ON',
        '-DCMAKE_BUILD_TYPE=Release',
        '-DCMAKE_INSTALL_PREFIX={}'.format(output_path),
        '-DCMAKE_LIBRARY_OUTPUT_DIRECTORY={}'.format(output_path),
        '-DPYTHON_EXECUTABLE={}'.format(sys.executable),
    ]

    if is_windows:
        cmake_args += \
            ['-DCMAKE_LIBRARY_OUTPUT_DIRECTORY_RELEASE={}'.format(output_path)]
        if sys.maxsize > 2**32:
            cmake_args += ['-A', 'x64']

    cmd = ['cmake', '-B', _build_dir] + cmake_args
    errno = subprocess.call(cmd)
    assert errno == 0, 'Configure failed'

    # Build
    cmd = [
        'cmake', '--build', _build_dir,
        '--config', 'Release',
        '--target', 'install'
    ]
    errno = subprocess.call(cmd)
    assert errno == 0, 'Build failed'


class OpenCCExtension(setuptools.Extension, object):
    def __init__(self, name, sourcedir=''):
        setuptools.Extension.__init__(self, name, sources=[])
        self.sourcedir = os.path.abspath(sourcedir)


class BuildExtCommand(setuptools.command.build_ext.build_ext, object):
    def build_extension(self, ext):
        if self.inplace:
            output_path = os.path.join(_this_dir, 'python', 'opencc', 'clib')
        else:
            output_path = os.path.abspath(os.path.join(self.build_lib, 'opencc', 'clib'))
        if isinstance(ext, OpenCCExtension):
            build_libopencc(output_path)
        else:
            super(BuildExtCommand, self).build_extension(ext)


class BDistWheelCommand(wheel.bdist_wheel.bdist_wheel, object):
    """Custom bdsit_wheel command that will change
    default plat-name based on PEP 425 and PEP 513
    """

    @staticmethod
    def _determine_platform_tag():
        if sys.platform == 'win32':
            if 'amd64' in sys.version.lower():
                return 'win-amd64'
            return sys.platform

        if sys.platform == 'darwin':
            _, _, _, _, machine = os.uname()
            if machine == 'x86_64':
                return 'macosx-10.9-{}'.format(machine)
            if machine == 'arm64':
                return 'macosx-11.0-{}'.format(machine)
            else:
                raise NotImplementedError

        if os.name == 'posix':
            _, _, _, _, machine = os.uname()
            return 'manylinux2014-{}'.format(machine)

        warnings.warn(
            'Windows macos and linux are all not detected, '
            'Proper distribution name cannot be determined.')
        from distutils.util import get_platform
        return get_platform()

    def initialize_options(self):
        super(BDistWheelCommand, self).initialize_options()
        self.plat_name = self._determine_platform_tag()


packages = ['opencc', 'opencc.clib']

version_info = get_version_info()
author_info = get_author_info()

setup_requires = []
if not shutil.which('cmake'):
    setup_requires.append('cmake')

setuptools.setup(
    name='OpenCC',
    version=version_info,
    author=author_info[0],
    author_email=author_info[1],
    description=" Conversion between Traditional and Simplified Chinese",
    long_description=get_long_description(),
    long_description_content_type="text/markdown",
    url="https://github.com/BYVoid/OpenCC",

    packages=packages,
    package_dir={'opencc': 'python/opencc'},
    ext_modules=[OpenCCExtension('opencc.clib.opencc_clib', 'python')],
    cmdclass={
        'build_ext': BuildExtCommand,
        'bdist_wheel': BDistWheelCommand,
    },
    setup_requires=setup_requires,

    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Intended Audience :: Science/Research',
        'Natural Language :: Chinese (Simplified)',
        'Natural Language :: Chinese (Traditional)',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: Apache Software License',
        'Topic :: Scientific/Engineering',
        'Topic :: Software Development',
        'Topic :: Software Development :: Libraries',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Software Development :: Localization',
        'Topic :: Text Processing :: Linguistic',
        'Typing :: Typed',
    ],
    license='Apache License 2.0',
    keywords=['opencc', 'convert', 'chinese'],
)
